<?php

/**
 * @file
 * CDEK API elements.
 */

use CdekSDK\Requests\PvzListRequest;
use CdekSDK\Common\Pvz;

/**
 * Implements hook_element_info().
 *
 * @see cdek_api_element_info()
 */
function _cdek_api_element_info() {
  $types['cdek_select'] = [
    '#input' => TRUE,
    '#tree' => TRUE,
    '#value_callback' => 'cdek_select_element_value_callback',
    '#process' => ['cdek_select_element_process'],
    '#after_build' => ['cdek_select_element_after_build'],
    '#pre_render' => [
      'form_pre_render_conditional_form_element',
      'cdek_select_element_pre_render',
    ],
    '#theme_wrappers' => ['container__cdek_select'],
    '#cdek_request' => NULL,
    '#cdek_hide_country' => FALSE,
    '#cdek_hide_region' => FALSE,
    '#cdek_hide_city' => FALSE,
  ];
  return $types;
}

/**
 * Value callback for the cdek_select form element.
 *
 * @param array $element
 *   An associative array containing the properties of the element.
 * @param mixed $input
 *   The incoming input to populate the form element. If this is FALSE,
 *   the element's default value should be returned.
 *
 * @return mixed
 *   The value to assign to the element.
 */
function cdek_select_element_value_callback(array $element, $input = FALSE) {
  $value = [];

  if ($input === FALSE) {
    if (isset($element['#default_value'])) {
      // Trying to find the pickup point.
      $point = CdekApi::getInstance()->getPickupPoint($element['#default_value']);

      if ($point !== NULL) {
        $value = [
          'country' => $point->CountryCode,
          'region' => $point->RegionCode,
          'city' => $point->CityCode,
          'point' => $point->Code,
        ];
      }
    }
  }
  elseif (is_array($input)) {
    $value = $input;
  }
  return $value;
}

/**
 * Processes a cdek_select form element.
 *
 * @param array $element
 *   The form element to process.
 *
 * @return array
 *   The processed element.
 */
function cdek_select_element_process(array $element) {
  $value = is_array($element['#value']) ? $element['#value'] : [];
  $is_value_locked = FALSE;

  // Get the request object.
  $request = $element['#cdek_request'];
  if ($request instanceof PvzListRequest) {
    $request = clone $request;
  }
  else {
    $request = new PvzListRequest();
  }

  $params = $request->getParams();
  $element['#cdek_hide_country'] = $element['#cdek_hide_country'] || isset($params['cityid']) || isset($params['regionid']) || isset($params['countryid']);
  $element['#cdek_hide_region'] = $element['#cdek_hide_region'] || isset($params['cityid']) || isset($params['regionid']);
  $element['#cdek_hide_city'] = $element['#cdek_hide_city'] || isset($params['cityid']);

  // Generate a unique wrapper HTML ID.
  $ajax_wrapper_id = drupal_html_id('ajax-wrapper');
  $ajax_settings = [
    'callback' => 'cdek_select_element_ajax_update_callback',
    'options' => [
      'query' => [
        'element_parents' => implode('/', $element['#array_parents']),
      ],
    ],
    'wrapper' => $ajax_wrapper_id,
    'effect' => 'fade',
  ];

  if (!$element['#cdek_hide_country']) {
    // The country selection does not depend on other elements.
    $countries = CdekApi::getInstance()->getCountries($request) ?: [];
    $country = isset($value['country']) && isset($countries[$value['country']]) ? $value['country'] : '';

    // Element to select the country.
    $element['country'] = [
      '#type' => 'select',
      '#title' => t('Country'),
      '#options' => $countries,
      '#empty_value' => '',
      '#value' => $country,
      '#ajax' => $ajax_settings,
    ];

    // If the country is not yet selected, block further selection.
    // Otherwise, add the country to the parameters.
    if ($country === '') {
      $is_value_locked = TRUE;
    }
    else {
      $request->setCountryId($value['country']);
    }
  }

  if (!$element['#cdek_hide_region']) {
    // The region selection may depend on the selected country.
    $regions = !$is_value_locked ? (CdekApi::getInstance()->getRegions($request) ?: []) : [];
    $region = isset($value['region']) && isset($regions[$value['region']]) ? $value['region'] : '';

    // Element to select the region.
    $element['region'] = [
      '#type' => 'select',
      '#title' => t('Region'),
      '#options' => $regions,
      '#empty_value' => '',
      '#value' => $region,
      '#ajax' => $ajax_settings,
    ];

    // If the region is not yet selected, block further selection.
    // Otherwise, add the region to the parameters.
    if ($region === '') {
      $is_value_locked = TRUE;
    }
    else {
      $request->setRegionId($value['region']);
    }
  }

  if (!$element['#cdek_hide_city']) {
    // The city selection may depend on the selected country and region.
    $cities = !$is_value_locked ? (CdekApi::getInstance()->getCities($request) ?: []) : [];
    $city = isset($value['city']) && isset($cities[$value['city']]) ? $value['city'] : '';

    // Element to select the city.
    $element['city'] = [
      '#type' => 'select',
      '#title' => t('City'),
      '#options' => $cities,
      '#empty_value' => '',
      '#value' => $city,
      '#ajax' => $ajax_settings,
    ];

    // If the city is not yet selected, block further selection.
    // Otherwise, add the city to the parameters.
    if ($city === '') {
      $is_value_locked = TRUE;
    }
    else {
      $request->setCityId($value['city']);
    }
  }

  // The point selection may depend on the selected country, region and city.
  $points = !$is_value_locked ? (CdekApi::getInstance()->getPickupPoints($request) ?: []) : [];
  $points = array_map('cdek_select_element_get_point_label', $points);
  $point = isset($value['point']) && isset($points[$value['point']]) ? $value['point'] : '';

  // Element to select the point.
  $element['point'] = [
    '#type' => 'select',
    '#title' => t('Point'),
    '#options' => $points,
    '#empty_value' => '',
    '#value' => $point,
  ];

  // Prefix and suffix used for Ajax replacement.
  if (isset($element['country']) || isset($element['region']) || isset($element['city'])) {
    $element['#prefix'] = '<div id="' . $ajax_wrapper_id . '">';
    $element['#suffix'] = '</div>';
  }
  return $element;
}

/**
 * After-build callback for the cdek_select form element.
 *
 * @param array $element
 *   The element structure.
 * @param array $form_state
 *   The current state of the form.
 *
 * @return array
 *   The element structure.
 */
function cdek_select_element_after_build(array $element, array &$form_state) {
  $element['#value'] = $element['point']['#value'];
  form_set_value($element, $element['#value'], $form_state);
  return $element;
}

/**
 * Pre-render callback for the cdek_select form element.
 *
 * @param array $element
 *   The element structure.
 *
 * @return array
 *   The element structure.
 */
function cdek_select_element_pre_render(array $element) {
  $element += ['#attributes' => []];

  // Set the correct html ID.
  if (isset($element['#id'])) {
    $element['#attributes']['id'] = $element['#id'];
  }
  $element['#attributes']['class'][] = 'form-cdek-select';
  return $element;
}

/**
 * Ajax callback to update cdek_select form element.
 *
 * @param array $form
 *   The form structure.
 *
 * @return array
 *   The element to be rendered.
 */
function cdek_select_element_ajax_update_callback(array $form) {
  // Get element parents from the request.
  $element_parents = explode('/', $_GET['element_parents']);
  // Sanitize element parents before using them.
  $element_parents = array_filter($element_parents, 'element_child');
  // Retrieve the element to be rendered.
  return drupal_array_get_nested_value($form, $element_parents);
}

/**
 * Gets the pickup point label.
 *
 * @param \CdekSDK\Common\Pvz $point
 *   The pickup point object.
 *
 * @return string
 *   The pickup point label.
 */
function cdek_select_element_get_point_label(Pvz $point) {
  return $point->Name;
}
